/**
 * sigma_view_actions.js
 * 
 * Handlers for viewing Sigma rule data and search results
 */
const logger = require('../../../utils/logger');
const { handleError } = require('../../../utils/error_handler');
const { getSigmaRuleYaml } = require('../../../services/sigma/sigma_details_service');
const { searchSigmaRules } = require('../../../services/sigma/sigma_search_service');
const { getYamlViewBlocks } = require('../../../blocks/sigma/sigma_view_yaml_block');
const { getSearchResultBlocks } = require('../../../blocks/sigma/sigma_search_results_block');
const { processRuleDetails } = require('./sigma_action_core');

const FILE_NAME = 'sigma_view_actions.js';

/**
 * Handle pagination actions (Previous, Next)
 * 
 * @param {Object} body - The action payload body
 * @param {Function} ack - Function to acknowledge the action
 * @param {Function} respond - Function to send response
 */
const handlePaginationAction = async (body, ack, respond) => {
  try {
    await ack();
    logger.debug(`${FILE_NAME}: Pagination action received: ${JSON.stringify(body.actions)}`);

    if (!body || !body.actions || !body.actions[0] || !body.actions[0].value) {
      logger.error(`${FILE_NAME}: Invalid pagination action payload: missing parameters`);
      await respond({
        text: 'Error: Could not process pagination request',
        replace_original: false
      });
      return;
    }

    // Parse the action value which contains our pagination parameters
    const action = body.actions[0];
    let valueData;

    try {
      valueData = JSON.parse(action.value);
    } catch (parseError) {
      await handleError(parseError, `${FILE_NAME}: Pagination value parsing`, respond, {
        replaceOriginal: false,
        customMessage: 'Error: Invalid pagination parameters'
      });
      return;
    }

    const { keyword, page, pageSize } = valueData;

    if (!keyword) {
      logger.warn(`${FILE_NAME}: Missing keyword in pagination action`);
      await respond({
        text: 'Error: Missing search keyword in pagination request',
        replace_original: false
      });
      return;
    }

    logger.info(`${FILE_NAME}: Processing pagination request for "${keyword}" (page ${page}, size ${pageSize})`);

    // Perform the search with the new pagination parameters
    const searchResult = await searchSigmaRules(keyword, page, pageSize);

    if (!searchResult.success) {
      logger.error(`${FILE_NAME}: Search failed during pagination: ${searchResult.message}`);
      await respond({
        text: `Error: ${searchResult.message}`,
        replace_original: false
      });
      return;
    }

    // Generate the updated blocks for the search results
    let blocks;
    try {
      blocks = getSearchResultBlocks(
        keyword,
        searchResult.results,
        searchResult.pagination
      );
    } catch (blockError) {
      await handleError(blockError, `${FILE_NAME}: Pagination block generation`, respond, {
        replaceOriginal: false,
        customMessage: `Error generating results view: ${blockError.message}`
      });
      return;
    }

    // Return the response that will update the original message
    await respond({
      blocks: blocks,
      replace_original: true
    });
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Pagination action handler`, respond, {
      replaceOriginal: false
    });
  }
};

/**
 * Register view-related action handlers
 * 
 * @param {Object} app - The Slack app instance
 */
const registerViewActions = (app) => {
  logger.info(`${FILE_NAME}: Registering view-related action handlers`);

  // Handle View YAML button clicks
  app.action('view_yaml', async ({ body, ack, respond }) => {
    logger.info(`${FILE_NAME}: VIEW_YAML ACTION TRIGGERED`);
    try {
      await ack();
      logger.debug(`${FILE_NAME}: View YAML action received: ${JSON.stringify(body.actions)}`);

      if (!body || !body.actions || !body.actions[0] || !body.actions[0].value) {
        logger.error(`${FILE_NAME}: Invalid action payload: missing rule ID`);
        await respond({
          text: 'Error: Could not determine which rule to get YAML for',
          replace_original: false
        });
        return;
      }

      // Extract rule ID from button value
      // Handle both formats: direct ID from search results or view_yaml_{ruleId} from details view
      let ruleId = body.actions[0].value;
      if (ruleId.startsWith('view_yaml_')) {
        ruleId = ruleId.replace('view_yaml_', '');
      }

      logger.info(`${FILE_NAME}: View YAML button clicked for rule: ${ruleId}`);

      // Get Sigma rule YAML
      const result = await getSigmaRuleYaml(ruleId);
      logger.debug(`${FILE_NAME}: YAML retrieval result: ${JSON.stringify(result, null, 2)}`);

      if (!result.success) {
        logger.error(`${FILE_NAME}: Rule YAML retrieval failed: ${result.message}`);
        await respond({
          text: `Error: ${result.message}`,
          replace_original: false
        });
        return;
      }

      logger.info(`${FILE_NAME}: Rule ${ruleId} YAML retrieved successfully via button click`);

      // Use the module to generate blocks
      const blocks = getYamlViewBlocks(ruleId, result.yaml || '');

      // Respond with the YAML content
      await respond({
        blocks: blocks,
        replace_original: false
      });
    } catch (error) {
      await handleError(error, `${FILE_NAME}: View YAML action`, respond, {
        replaceOriginal: false
      });
    }
  });

  // Handle "View Rule Details" button clicks from search results
  app.action('view_rule_details', async ({ body, ack, respond }) => {
    logger.info(`${FILE_NAME}: VIEW_RULE_DETAILS ACTION TRIGGERED`);
    try {
      await ack();
      logger.debug(`${FILE_NAME}: View Rule Details action received: ${JSON.stringify(body.actions)}`);

      if (!body || !body.actions || !body.actions[0] || !body.actions[0].value) {
        logger.error(`${FILE_NAME}: Invalid action payload: missing rule ID`);
        await respond({
          text: 'Error: Could not determine which rule to explain',
          replace_original: false
        });
        return;
      }

      const ruleId = body.actions[0].value;
      logger.info(`${FILE_NAME}: Rule details button clicked for rule ID: ${ruleId}`);

      // Inform user we're processing
      await respond({
        text: `Processing details for rule ${ruleId}...`,
        replace_original: false,
        response_type: 'ephemeral'
      });

      await processRuleDetails(ruleId, respond, false, 'in_channel');
    } catch (error) {
      await handleError(error, `${FILE_NAME}: View rule details action`, respond, {
        replaceOriginal: false
      });
    }
  });

  // Handle pagination button clicks
  app.action('search_prev_page', async ({ body, ack, respond }) => {
    await handlePaginationAction(body, ack, respond);
  });

  app.action('search_next_page', async ({ body, ack, respond }) => {
    await handlePaginationAction(body, ack, respond);
  });

  logger.info(`${FILE_NAME}: All view action handlers registered successfully`);
};

module.exports = {
  registerViewActions,
  handlePaginationAction
};