//
// sigma_converter_service.js
// converts Sigma rules to a structured object 
//
const logger = require('../../utils/logger');
const yaml = require('js-yaml');
const { findRuleById } = require('../../sigma_db/queries');

const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Convert a Sigma rule to a structured object
 * Can be called with either a rule object or a rule ID
 *
 * @param {Object|String} input - Either a raw rule object or a rule ID
 * @param {Object} [config] - Optional configuration
 * @returns {Promise<Object>} Converted rule or result object
 */
async function convertSigmaRule(input, config = null) {
  // Check if we're dealing with a rule ID (string)
  if (typeof input === 'string') {
    try {
      const ruleId = input;
      logger.info(`Converting rule by ID: ${ruleId}`);
      // Find the rule in database
      const rawRule = await findRuleById(ruleId);
      if (!rawRule) {
        logger.warn(`Rule with ID ${ruleId} not found`);
        return {
          success: false,
          message: `Rule with ID ${ruleId} not found`
        };
      }
      
      // Debug: Log what we found
      logger.debug(`Retrieved rule ${ruleId} from database: content ${rawRule.content ? 'present' : 'missing'}, parameters ${rawRule.parameters ? Object.keys(rawRule.parameters).length : 0}`);
      
      // Check if content is missing (flag set by findRuleById)
      if (rawRule.content_missing || !rawRule.content) {
        logger.warn(`Rule with ID ${ruleId} has missing content, attempting to build from parameters`);
        
        // Try to build from parameters
        if (rawRule.parameters && Object.keys(rawRule.parameters).length > 0) {
          const builtRule = buildRuleFromParameters(rawRule);
          
          if (builtRule) {
            logger.info(`Successfully built rule ${ruleId} from parameters`);
            return {
              success: true,
              rule: builtRule,
              built_from_parameters: true
            };
          }
        }
        
        logger.warn(`Could not build rule ${ruleId} from parameters, returning placeholder`);
        return {
          success: true,
          rule: {
            id: ruleId,
            title: 'Rule Found But Content Missing',
            description: `The rule with ID ${ruleId} exists in the database, but its content field is empty. This may indicate a problem with the rule import process.`,
            author: 'Unknown',
            level: 'unknown',
            status: 'unknown',
            logsource: {},
            detection: { condition: 'Content missing' },
            falsepositives: ['N/A - Content missing'],
            tags: ['error', 'missing-content'],
            references: [],
            file_path: rawRule.file_path || 'unknown'
          }
        };
      }
      
      // Process the raw rule
      const processedRule = processRuleContent(rawRule);
      if (!processedRule) {
        return {
          success: false,
          message: `Failed to process rule with ID ${ruleId}`
        };
      }
      
      logger.debug(`Processing rule content for ${rawRule.id}:`);

      return {
        success: true,
        rule: processedRule
      };
    } catch (error) {
      logger.error(`Error converting rule by ID: ${error.message}`);
      return {
        success: false,
        message: `Error converting rule: ${error.message}`
      };
    }
  } else {
    try {
      if (!input) {
        return {
          success: false,
          message: 'No rule data provided'
        };
      }
      
      // Check for missing content
      if (!input.content) {
        logger.warn('Rule object has missing content, attempting to build from parameters');
        
        // Try to build from parameters
        if (input.parameters && Object.keys(input.parameters).length > 0) {
          const builtRule = buildRuleFromParameters(input);
          
          if (builtRule) {
            logger.info(`Successfully built rule ${input.id} from parameters`);
            return {
              success: true,
              rule: builtRule,
              built_from_parameters: true
            };
          }
        }
        
        logger.warn(`Could not build rule from parameters, returning placeholder`);
        return {
          success: true,
          rule: {
            id: input.id || 'unknown',
            title: 'Rule Found But Content Missing',
            description: 'The rule exists in the database, but its content field is empty. This may indicate a problem with the rule import process.',
            author: 'Unknown',
            level: 'unknown',
            status: 'unknown',
            logsource: {},
            detection: { condition: 'Content missing' },
            falsepositives: ['N/A - Content missing'],
            tags: ['error', 'missing-content'],
            references: [],
            file_path: input.file_path || 'unknown'
          }
        };
      }
      
      const processedRule = processRuleContent(input);
      if (!processedRule) {
        return {
          success: false,
          message: 'Failed to process rule object'
        };
      }
      
      return {
        success: true,
        rule: processedRule
      };
    } catch (error) {
      logger.error(`Error processing rule object: ${error.message}`);
      return {
        success: false,
        message: `Error processing rule: ${error.message}`
      };
    }
  }
}

/**
 * Process rule content into a structured object
 * @param {Object} rawRule - The raw rule object
 * @returns {Object|null} Processed rule object
 */
function processRuleContent(rawRule) {
  if (!rawRule) {
    logger.warn('Cannot convert rule: rule object is null');
    return null;
  }
  
  if (!rawRule.content) {
    logger.warn('Cannot convert rule: missing content in rule data');
    
    // Check if we have parameters and try to build from them
    if (rawRule.parameters && Object.keys(rawRule.parameters).length > 0) {
      logger.info(`Attempting to build rule ${rawRule.id} from parameters`);
      return buildRuleFromParameters(rawRule);
    }
    
    return {
      id: rawRule.id || 'unknown',
      title: 'Error: Missing Rule Content',
      description: 'The rule content could not be found in the database. This may indicate a problem with the rule import process or a corruption in the database.',
      level: 'unknown',
      file_path: rawRule.file_path || 'unknown',
      falsepositives: ['N/A - Content missing'],
      tags: ['error', 'missing-content'],
      references: [],
      detection: { condition: 'Content missing' }
    };
  }
  
  try {
    // Parse the YAML content
    let parsedRule;
    try {
      // Log the content for debugging
      logger.debug(`Parsing YAML content for rule ${rawRule.id}, content length: ${rawRule.content.length}`);
      
      // Try different YAML parsing approaches
      try {
        parsedRule = yaml.load(rawRule.content);
      } catch (yamlError) {
        logger.warn(`Standard YAML parsing failed for ${rawRule.id}: ${yamlError.message}`);
        
        // Try with more tolerant parsing
        try {
          // Try multi-document loading
          const docs = [];
          yaml.loadAll(rawRule.content, (doc) => {
            if (doc) docs.push(doc);
          });
          
          if (docs.length > 0) {
            parsedRule = docs[0]; // Take the first document
            logger.debug(`Multi-document YAML parsing succeeded for ${rawRule.id}, found ${docs.length} documents`);
          } else {
            throw new Error('No documents found in multi-document parse');
          }
        } catch (multiError) {
          logger.warn(`Multi-document YAML parsing failed for ${rawRule.id}: ${multiError.message}`);
          
          // Last resort: manual extraction of key fields
          parsedRule = extractFieldsManually(rawRule.content, rawRule.id);
        }
      }
      
      if (!parsedRule) {
        logger.warn(`Rule parsing resulted in null object for ID: ${rawRule.id}`);
        parsedRule = {};
      }
    } catch (yamlError) {
      logger.error(`YAML parsing error: ${yamlError.message}`);
      logger.debug(`Problematic content (first 200 chars): ${rawRule.content.substring(0, 200)}`);
      parsedRule = {};
    }
    
    // Create a new object combining database fields and parsed content
    const convertedRule = {
      id: rawRule.id || parsedRule.id || 'unknown',
      title: parsedRule.title || 'Untitled Rule',
      description: parsedRule.description || 'No description provided',
      author: parsedRule.author || 'Unknown',
      level: parsedRule.level || 'unknown',
      status: parsedRule.status || 'unknown',
      logsource: parsedRule.logsource || {},
      detection: parsedRule.detection || {},
      falsepositives: parsedRule.falsepositives || [],
      tags: parsedRule.tags || [],
      references: parsedRule.references || [],
      file_path: rawRule.file_path || 'unknown'
    };
    
    logger.info(`Successfully converted rule ${convertedRule.id}`);
    return convertedRule;
  } catch (error) {
    logger.error(`Error parsing rule: ${error.message}`);
    return {
      id: rawRule.id || 'unknown',
      title: 'Error: Could not parse rule',
      description: `Error parsing rule: ${error.message}`,
      level: 'unknown',
      file_path: rawRule.file_path || 'unknown',
      falsepositives: [],
      tags: ['error', 'parse-error'],
      references: [],
      detection: { condition: 'Parse error' }
    };
  }
}

/**
 * Manual extraction of key fields from YAML content when parsing fails
 * @param {string} content - The raw YAML content
 * @param {string} ruleId - The rule ID
 * @returns {Object} Extracted fields
 */
function extractFieldsManually(content, ruleId) {
  logger.debug(`Attempting manual field extraction for rule ${ruleId}`);
  
  const result = {
    id: ruleId
  };
  
  // Simple regex patterns to extract common fields
  const patterns = {
    title: /title:\s*(.+)$/m,
    description: /description:\s*(.+)$/m,
    author: /author:\s*(.+)$/m,
    level: /level:\s*(.+)$/m,
    status: /status:\s*(.+)$/m
  };
  
  // Extract fields using regex
  Object.entries(patterns).forEach(([field, pattern]) => {
    const match = content.match(pattern);
    if (match && match[1]) {
      result[field] = match[1].trim();
    }
  });
  
  logger.debug(`Manual extraction found ${Object.keys(result).length - 1} fields for rule ${ruleId}`);
  
  return result;
}

/**
 * Build a rule object from parameters when content is missing
 * @param {Object} rawRule - The raw rule object with parameters
 * @returns {Object} Reconstructed rule object
 */
function buildRuleFromParameters(rawRule) {
  logger.info(`Building rule ${rawRule.id} from parameters`);
  
  if (!rawRule || !rawRule.parameters) {
    logger.warn(`Cannot build rule: missing parameters for rule ${rawRule ? rawRule.id : 'unknown'}`);
    return null;
  }
  
  logger.debug(`Found ${Object.keys(rawRule.parameters).length} parameters for rule ${rawRule.id}`);
  
  // Initialize a new rule object with essential properties
  const reconstructedRule = {
    id: rawRule.id,
    title: rawRule.parameters.title || 'Unknown Title',
    description: rawRule.parameters.description || 'No description available',
    author: rawRule.parameters.author || 'Unknown',
    file_path: rawRule.file_path || 'unknown',
    level: rawRule.parameters.level || 'unknown',
    status: rawRule.parameters.status || 'unknown',
    logsource: {},
    detection: { condition: rawRule.parameters['detection.condition'] || 'unknown' },
    falsepositives: [],
    tags: [],
    references: []
  };
  
  // Process parameters to rebuild nested objects
  Object.entries(rawRule.parameters).forEach(([key, value]) => {
    // Handle array parameters
    if (key === 'falsepositives' || key === 'tags' || key === 'references') {
      if (Array.isArray(value)) {
        reconstructedRule[key] = value;
      } else if (typeof value === 'string') {
        // Try to parse JSON string arrays
        try {
          const parsed = JSON.parse(value);
          if (Array.isArray(parsed)) {
            reconstructedRule[key] = parsed;
          } else {
            reconstructedRule[key] = [value];
          }
        } catch (e) {
          reconstructedRule[key] = [value];
        }
      }
    }
    // Handle logsource properties
    else if (key.startsWith('logsource.')) {
      const prop = key.substring('logsource.'.length);
      reconstructedRule.logsource[prop] = value;
    }
    // Handle detection properties
    else if (key.startsWith('detection.') && key !== 'detection.condition') {
      const prop = key.substring('detection.'.length);
      const parts = prop.split('.');
      
      let current = reconstructedRule.detection;
      for (let i = 0; i < parts.length - 1; i++) {
        if (!current[parts[i]]) {
          current[parts[i]] = {};
        }
        current = current[parts[i]];
      }
      
      current[parts[parts.length - 1]] = value;
    }
  });
  
  logger.debug(`Reconstructed rule structure for ${rawRule.id}: ${JSON.stringify({
    id: reconstructedRule.id,
    title: reconstructedRule.title,
    fields: Object.keys(reconstructedRule)
  })}`);
  
  return reconstructedRule;
}

/**
 * Extract a readable condition string from a rule
 * @param {Object} rule - The converted rule object
 * @returns {String} Human-readable condition
 */
function extractDetectionCondition(rule) {
  if (!rule) {
    return 'No rule data available';
  }
  
  if (!rule.detection) {
    return 'No detection information available';
  }
  
  if (!rule.detection.condition) {
    return 'No condition specified';
  }
  
  return rule.detection.condition;
}

module.exports = {
  convertSigmaRule,
  extractDetectionCondition,
  buildRuleFromParameters
};