/**
 * sigma_repository_service.js
 * 
 * This service manages the Sigma rule repository and database updates.
 * It provides functions to clone/update the repository and run the database
 * initialization script.
 */
const { spawn } = require('child_process');
const util = require('util');
const { exec } = require('child_process');
const fs = require('fs');
const path = require('path');
const { SIGMA_REPO_DIR } = require('../../config/appConfig');
const appConfig = require('../../config/appConfig');
const logger = require('../../utils/logger');

const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

// Promisify exec for async/await usage
const execPromise = util.promisify(exec);

/**
 * Clones or updates the Sigma repository
 * Creates the repository directory if it doesn't exist
 * 
 * @returns {Promise<boolean>} Success status of the operation
 */
async function updateSigmaRepo() {
  logger.debug(`${FILE_NAME}: Starting Sigma repository update process`);
  
  try {
    // Ensure the parent directory exists
    const parentDir = path.dirname(SIGMA_REPO_DIR);
    if (!fs.existsSync(parentDir)) {
      logger.debug(`${FILE_NAME}: Creating parent directory: ${parentDir}`);
      fs.mkdirSync(parentDir, { recursive: true });
    }
    
    if (!fs.existsSync(SIGMA_REPO_DIR)) {
      logger.info(`${FILE_NAME}: Cloning Sigma repository...`);
      
      // Read config to get repo URL
      const repoUrl = appConfig.SIGMA_REPO_CONFIG.url;
      if (!repoUrl) {
        throw new Error('Repository URL not found in configuration');
      }
      
      logger.debug(`${FILE_NAME}: Using repository URL: ${repoUrl}`);
      const cloneResult = await execPromise(`git clone ${repoUrl} ${SIGMA_REPO_DIR}`);
      
      logger.debug(`${FILE_NAME}: Clone output: ${cloneResult.stdout}`);
    } else {
      logger.info(`${FILE_NAME}: Updating existing Sigma repository...`);
      
      // Check if it's actually a git repository
      if (!fs.existsSync(path.join(SIGMA_REPO_DIR, '.git'))) {
        logger.warn(`${FILE_NAME}: Directory exists but is not a git repository: ${SIGMA_REPO_DIR}`);
        throw new Error('Directory exists but is not a git repository');
      }
      
      const pullResult = await execPromise(`cd ${SIGMA_REPO_DIR} && git pull`);
      logger.debug(`${FILE_NAME}: Pull output: ${pullResult.stdout}`);
    }
    
    logger.info(`${FILE_NAME}: Sigma repository is up-to-date`);
    return true;
  } catch (error) {
    logger.error(`${FILE_NAME}: Error updating Sigma repository: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    return false;
  }
}

/**
 * Updates the Sigma database by running the initialization script
 * Spawns a child process to run the database initialization
 * 
 * @returns {Promise<boolean>} Success status of the operation
 */
async function updateSigmaDatabase() {
  logger.info(`${FILE_NAME}: Starting database update process`);
  
  return new Promise((resolve, reject) => {
    const scriptPath = path.join(__dirname, '..', '..', 'db', 'init-sigma-db.js');
    
    // Verify the script exists before trying to run it
    if (!fs.existsSync(scriptPath)) {
      logger.error(`${FILE_NAME}: Database initialization script not found at: ${scriptPath}`);
      reject(new Error(`Database initialization script not found at: ${scriptPath}`));
      return;
    }
    
    logger.info(`${FILE_NAME}: Running database update script: ${scriptPath}`);
    
    const updateProcess = spawn('node', [scriptPath], {
      stdio: 'pipe' // Capture output instead of inheriting
    });
    
    // Capture and log stdout
    updateProcess.stdout.on('data', (data) => {
      logger.debug(`${FILE_NAME}: DB Update stdout: ${data.toString().trim()}`);
    });
    
    // Capture and log stderr
    updateProcess.stderr.on('data', (data) => {
      logger.warn(`${FILE_NAME}: DB Update stderr: ${data.toString().trim()}`);
    });
    
    updateProcess.on('close', (code) => {
      if (code === 0) {
        logger.info(`${FILE_NAME}: Database update completed successfully`);
        resolve(true);
      } else {
        logger.error(`${FILE_NAME}: Database update failed with exit code ${code}`);
        reject(new Error(`Update failed with exit code ${code}`));
      }
    });
    
    updateProcess.on('error', (err) => {
      logger.error(`${FILE_NAME}: Failed to start database update process: ${err.message}`);
      reject(err);
    });
  });
}

/**
 * Checks the status of the Sigma repository
 * Returns information about the repository including last commit
 * 
 * @returns {Promise<Object>} Repository status information
 */
async function getSigmaRepoStatus() {
  logger.debug(`${FILE_NAME}: Checking Sigma repository status`);
  
  try {
    if (!fs.existsSync(SIGMA_REPO_DIR)) {
      logger.warn(`${FILE_NAME}: Sigma repository directory does not exist: ${SIGMA_REPO_DIR}`);
      return {
        exists: false,
        message: 'Repository has not been cloned yet'
      };
    }
    
    // Check if it's a git repository
    if (!fs.existsSync(path.join(SIGMA_REPO_DIR, '.git'))) {
      logger.warn(`${FILE_NAME}: Directory exists but is not a git repository: ${SIGMA_REPO_DIR}`);
      return {
        exists: true,
        isRepo: false,
        message: 'Directory exists but is not a git repository'
      };
    }
    
    // Get last commit info
    const lastCommitInfo = await execPromise(`cd ${SIGMA_REPO_DIR} && git log -1 --format="%h|%an|%ad|%s"`);
    const [hash, author, date, subject] = lastCommitInfo.stdout.trim().split('|');
    
    // Get branch info
    const branchInfo = await execPromise(`cd ${SIGMA_REPO_DIR} && git branch --show-current`);
    const currentBranch = branchInfo.stdout.trim();
    
    return {
      exists: true,
      isRepo: true,
      lastCommit: {
        hash,
        author,
        date,
        subject
      },
      branch: currentBranch,
      path: SIGMA_REPO_DIR
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error getting repository status: ${error.message}`);
    return {
      exists: true,
      error: error.message
    };
  }
}

module.exports = {
  updateSigmaRepo,
  updateSigmaDatabase,
  getSigmaRepoStatus
};