/**
 * query-builders.js
 * Helper functions for building SQL queries
 */

const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Build the SQL query for complex search based on parsed conditions
 * 
 * @param {Object} parsedQuery - The parsed query object
 * @param {number} limit - Results limit
 * @param {number} offset - Results offset
 * @returns {Object} Object with SQL query, count query, and parameters
 */
function buildComplexSqlQuery(parsedQuery, limit, offset) {
  const { conditions, operator } = parsedQuery;
  const params = [];

  // Start building the primary table selection
  let sqlSelectPart = `
    SELECT DISTINCT r.id as rule_id, 
      (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'title' LIMIT 1) as title
    FROM sigma_rules r
  `;

  // Build WHERE clause based on conditions
  let whereClauses = [];
  let joinIdx = 0;

  for (const condition of conditions) {
    let whereClause = '';

    switch (condition.field) {
      case 'title':
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'title'
          AND INSTR(LOWER(p${joinIdx}.param_value), LOWER(?)) > 0
        )`;
        params.push(condition.value);
        break;

      case 'description':
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'description'
          AND INSTR(LOWER(p${joinIdx}.param_value), LOWER(?)) > 0
        )`;
        params.push(condition.value);
        break;

      case 'logsource':
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'logsource'
          AND INSTR(LOWER(p${joinIdx}.param_value), LOWER('"${condition.subfield}":"${condition.value}"')) > 0
        )`;
        break;

      case 'tags':
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'tags'
          AND INSTR(LOWER(p${joinIdx}.param_value), LOWER(?)) > 0
        )`;
        params.push(condition.value);
        break;

      case 'date':
        joinIdx++;
        const dateOperator = condition.operator === 'after' ? '>' :
          condition.operator === 'before' ? '<' : '=';
        whereClause = `r.date ${dateOperator} date(?)`;
        params.push(condition.value);
        break;

      case 'author':
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'author'
          AND INSTR(LOWER(p${joinIdx}.param_value), LOWER(?)) > 0
        )`;
        params.push(condition.value);
        break;

      case 'level':
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'level'
          AND LOWER(p${joinIdx}.param_value) = LOWER(?)
        )`;
        params.push(condition.value);
        break;

      case 'id':
        whereClause = `LOWER(r.id) = LOWER(?)`;
        params.push(condition.value);
        break;

      case 'keyword':
      default:
        // Default to searching in title
        joinIdx++;
        whereClause = `EXISTS (
          SELECT 1 FROM rule_parameters p${joinIdx}
          WHERE p${joinIdx}.rule_id = r.id
          AND p${joinIdx}.param_name = 'title'
          AND INSTR(LOWER(p${joinIdx}.param_value), LOWER(?)) > 0
        )`;
        params.push(condition.value);
        break;
    }

    if (whereClause) {
      whereClauses.push(whereClause);
    }
  }

  // Combine the WHERE clauses with the appropriate operator
  let whereStatement = '';
  if (whereClauses.length > 0) {
    const combiner = operator === 'AND' ? ' AND ' : ' OR ';
    whereStatement = `WHERE ${whereClauses.join(combiner)}`;
  }

  // Complete queries
  const sqlQuery = `
    ${sqlSelectPart}
    ${whereStatement}
    ORDER BY rule_id
    LIMIT ? OFFSET ?
  `;

  const sqlCountQuery = `
    SELECT COUNT(DISTINCT r.id) as count
    FROM sigma_rules r
    ${whereStatement}
  `;

  // Add pagination parameters
  params.push(limit);
  params.push(offset);

  return { sqlQuery, sqlCountQuery, params };
}

/**
 * Build FTS query and WHERE clause from parsed query conditions
 * 
 * @param {Object} parsedQuery - The parsed query object
 * @returns {Object} Object with FTS query, additional WHERE clause, and parameters
 */
function buildComplexFtsQuery(parsedQuery) {
  const { conditions, operator } = parsedQuery;

  // Separate text search conditions from other conditions
  const textConditions = [];
  const nonTextConditions = [];

  for (const condition of conditions) {
    switch (condition.field) {
      case 'title':
      case 'description':
      case 'author':
      case 'tags':
      case 'keyword':
        // These can be handled by FTS directly
        textConditions.push(condition);
        break;
      default:
        // These need additional WHERE clauses
        nonTextConditions.push(condition);
        break;
    }
  }

  // Build FTS MATCH query
  let ftsQueryParts = [];

  for (const condition of textConditions) {
    let fieldPrefix = '';

    // Add field-specific prefix if available
    if (condition.field !== 'keyword') {
      fieldPrefix = `${condition.field}:`;
    }

    // Add wildcard for partial matching if not already present
    let value = condition.value.trim();
    if (!value.endsWith('*')) {
      value = `${value}*`;
    }

    ftsQueryParts.push(`${fieldPrefix}${value}`);
  }

  // If no text conditions, use a match-all query
  const ftsQuery = ftsQueryParts.length > 0
    ? ftsQueryParts.join(operator === 'AND' ? ' AND ' : ' OR ')
    : '*';

  // Build additional WHERE clauses for non-text conditions
  let whereClauseParts = [];
  const params = [];

  for (const condition of nonTextConditions) {
    switch (condition.field) {
      case 'date':
        const dateOperator = condition.operator === 'after' ? '>' :
          condition.operator === 'before' ? '<' : '=';
        whereClauseParts.push(`date ${dateOperator} date(?)`);
        params.push(condition.value);
        break;

      case 'level':
        whereClauseParts.push(`level = ?`);
        params.push(condition.value);
        break;

      case 'logsource':
        whereClauseParts.push(`logsource LIKE ?`);
        params.push(`%${condition.subfield}%${condition.value}%`);
        break;

      case 'id':
        whereClauseParts.push(`rule_id = ?`);
        params.push(condition.value);
        break;
    }
  }

  // Combine WHERE clauses
  const whereClause = whereClauseParts.length > 0
    ? whereClauseParts.join(operator === 'AND' ? ' AND ' : ' OR ')
    : '';

  return { ftsQuery, whereClause, params };
}

module.exports = {
  buildComplexSqlQuery,
  buildComplexFtsQuery
};