/**
 * simple-search.js
 * Functions for basic search of Sigma rules
 */

const { getDbConnection } = require('../sigma_db_connection');
const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

// Import FTS functions - need to use relative path for proper circular dependency handling
const { checkFtsAvailable, searchRulesFTS } = require('./fts-search');

/**
 * Search for Sigma rules by keyword in rule titles
 * Performs a case-insensitive search and returns matching rules with pagination
 * 
 * @param {string} keyword - The keyword to search for
 * @param {number} limit - Maximum number of results to return (default: 10)
 * @param {number} offset - Number of results to skip (for pagination, default: 0)
 * @returns {Promise<Object>} Object with results array and total count
 */
async function searchRules(keyword, limit = 10, offset = 0) {
  if (!keyword) {
    logger.warn(`${FILE_NAME}: Empty search keyword provided`);
    return { results: [], totalCount: 0 };
  }

  // Sanitize keyword to prevent SQL injection
  const sanitizedKeyword = keyword.replace(/'/g, "''");
  logger.info(`${FILE_NAME}: Searching for rules with keyword in title: ${sanitizedKeyword} (limit: ${limit}, offset: ${offset})`);

  let db;
  try {
    // Make sure we properly await the DB connection
    db = await getDbConnection();
    logger.debug(`${FILE_NAME}: Database connection established for search`);

    // Use FTS5 for faster searching if available
    const ftsAvailable = await checkFtsAvailable(db);

    if (ftsAvailable) {
      logger.debug(`${FILE_NAME}: Using FTS5 for keyword search`);
      return searchRulesFTS(keyword, limit, offset);
    }

    // If FTS5 is not available, use the legacy search method
    logger.debug(`${FILE_NAME}: FTS5 not available, using legacy search method`);

    // First get the total count of matching rules (for pagination info)
    const countQuery = `
      SELECT COUNT(*) as count
      FROM rule_parameters
      WHERE param_name = 'title'
      AND INSTR(LOWER(param_value), LOWER(?)) > 0
    `;

    const countResult = await new Promise((resolve, reject) => {
      db.get(countQuery, [sanitizedKeyword], (err, row) => {
        if (err) {
          logger.error(`${FILE_NAME}: Count query error: ${err.message}`);
          reject(err);
        } else {
          resolve(row || { count: 0 });
        }
      });
    });

    const totalCount = countResult.count;
    logger.debug(`${FILE_NAME}: Total matching rules for "${sanitizedKeyword}": ${totalCount}`);

    // Use parameterized query instead of string interpolation for better security
    const instrQuery = `
      SELECT rule_id, param_value AS title
      FROM rule_parameters
      WHERE param_name = 'title'
      AND INSTR(LOWER(param_value), LOWER(?)) > 0
      LIMIT ? OFFSET ?
    `;

    const results = await new Promise((resolve, reject) => {
      db.all(instrQuery, [sanitizedKeyword, limit, offset], (err, rows) => {
        if (err) {
          logger.error(`${FILE_NAME}: Search query error: ${err.message}`);
          reject(err);
        } else {
          logger.debug(`${FILE_NAME}: Search query returned ${rows ? rows.length : 0} results`);
          resolve(rows || []);
        }
      });
    });

    logger.debug(`${FILE_NAME}: Search results page for keyword "${sanitizedKeyword}": ${results.length} matches (page ${Math.floor(offset / limit) + 1})`);

    return {
      results: results.map(r => ({ id: r.rule_id, title: r.title })),
      totalCount
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in search operation: ${error.message}`);
    logger.debug(`${FILE_NAME}: Search error stack: ${error.stack}`);
    return { results: [], totalCount: 0 };
  } finally {
    // Make sure we properly close the connection
    if (db) {
      try {
        await new Promise((resolve) => db.close(() => resolve()));
        logger.debug(`${FILE_NAME}: Database connection closed after search operation`);
      } catch (closeError) {
        logger.error(`${FILE_NAME}: Error closing database connection after search: ${closeError.message}`);
      }
    }
  }
}

module.exports = {
  searchRules
};