/**
 * cli_formatters.js
 *
 * Dedicated formatters for CLI output of various data types
 * Converts raw data into formatted CLI-friendly displays
 */
const chalk = require('chalk');

/**
 * Wraps text at specified length
 * @param {string} text - Text to wrap
 * @param {number} maxLength - Maximum line length
 * @returns {string} Wrapped text
 */
function wrapText(text, maxLength = 80) {
  if (!text || typeof text !== 'string') {
    return text;
  }
  
  if (text.length <= maxLength) {
    return text;
  }
  
  const words = text.split(' ');
  let wrappedText = '';
  let currentLine = '';
  
  words.forEach(word => {
    // If adding this word would exceed max length, start a new line
    if ((currentLine + word).length + 1 > maxLength) {
      wrappedText += currentLine.trim() + '\n';
      currentLine = word + ' ';
    } else {
      currentLine += word + ' ';
    }
  });
  
  // Add the last line
  wrappedText += currentLine.trim();
  
  return wrappedText;
}

/**
 * Format Sigma rule details for CLI display
 * @param {Object} ruleDetails - The rule details to format
 * @returns {Object} Formatted rule details for CLI display
 */
function formatSigmaDetails(ruleDetails) {
  if (!ruleDetails) {
    return null;
  }
  
  // Create a flattened object for display in CLI table format
  const formattedDetails = {
    'ID': ruleDetails.id || 'Unknown',
    'Title': wrapText(ruleDetails.title || 'Untitled Rule', 80),
    'Description': wrapText(ruleDetails.description || 'No description provided', 80),
    'Author': ruleDetails.author || 'Unknown author',
    'Severity': ruleDetails.severity || 'Unknown',
    'Detection': wrapText(ruleDetails.detectionExplanation || 'No detection specified', 80),
    'False Positives': wrapText(Array.isArray(ruleDetails.falsePositives) ? 
                        ruleDetails.falsePositives.join(', ') : 'None specified', 80),
    'Tags': wrapText(Array.isArray(ruleDetails.tags) ? 
            ruleDetails.tags.join(', ') : 'None', 80),
    'References': wrapText(Array.isArray(ruleDetails.references) ? 
                  ruleDetails.references.join(', ') : 'None', 80)
  };
  
  return formattedDetails;
}

/**
 * Format Sigma statistics for CLI display
 * 
 * @param {Object} stats - The statistics object
 * @returns {Object} Formatted stats ready for CLI display
 */
function formatSigmaStats(stats) {
  if (!stats) {
    return { error: 'No statistics data available' };
  }
  
  // Format date
  const formatDate = (dateString) => {
    if (!dateString) return 'Unknown';
    try {
      const date = new Date(dateString);
      return date.toLocaleString();
    } catch (error) {
      return dateString;
    }
  };

  // Create a simplified object suitable for table display
  const formattedStats = {
    'Last Update': formatDate(stats.lastUpdate),
    'Total Rules': stats.totalRules.toLocaleString(),
    'Database Health': `${stats.databaseHealth.contentPercentage}% Complete`,
    
    // OS breakdown
    'Windows Rules': stats.operatingSystems.windows.toLocaleString(),
    'Linux Rules': stats.operatingSystems.linux.toLocaleString(),
    'macOS Rules': stats.operatingSystems.macos.toLocaleString(),
    'Other OS Rules': stats.operatingSystems.other.toLocaleString(),
    
    // Add severity levels
    ...(stats.severityLevels || []).reduce((acc, level) => {
      const levelName = level.level 
        ? level.level.charAt(0).toUpperCase() + level.level.slice(1) 
        : 'Unknown';
      
      acc[`${levelName} Severity`] = level.count.toLocaleString();
      return acc;
    }, {})
  };
  
  // Add top MITRE tactics if available
  if (stats.mitreTactics && stats.mitreTactics.length > 0) {
    stats.mitreTactics.forEach((tactic, index) => {
      if (index < 5) { // Only include top 5 for brevity
        const formattedTactic = tactic.tactic
          .replace(/-/g, ' ')
          .split(' ')
          .map(word => word.charAt(0).toUpperCase() + word.slice(1))
          .join(' ');
          
        formattedStats[`MITRE: ${formattedTactic}`] = tactic.count.toLocaleString();
      }
    });
  }
    
  return formattedStats;
}

/**
 * Format Sigma search results for CLI display
 * 
 * @param {Object} searchResults - The search results object
 * @returns {Object} Formatted results ready for CLI display
 */
function formatSigmaSearchResults(searchResults) {
  if (!searchResults || !searchResults.results) {
    return { error: 'No search results available' };
  }
  
  // Return a structure with results and meta info
  return {
    results: searchResults.results.map(rule => ({
      id: rule.id || '',
      title: wrapText(rule.title || '', 60), // Use narrower width for table columns
      author: rule.author || 'Unknown',
      level: rule.level || 'medium'
    })),
    totalCount: searchResults.totalCount || 0
  };
}

module.exports = {
  formatSigmaStats,
  formatSigmaSearchResults,
  formatSigmaDetails,
  wrapText
};